<?php

namespace WPSecurityNinja\Plugin;

if (!defined('ABSPATH')) {
	exit;
}

/**
 * Wf_Sn_Visitor_log.
 *
 * @author  Lars Koudal <admin@wpsecurityninja.com>
 * @since   v0.0.1
 * @version v1.0.0    Tuesday, April 18th, 2023.
 * @global
 */
class Wf_Sn_Visitor_log
{


	/**
	 * init.
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0    Friday, May 13th, 2022.
	 * @access  public static
	 * @return  void
	 */
	public static function init()
	{
		add_action('admin_enqueue_scripts', array(__NAMESPACE__ . '\\Wf_Sn_Visitor_log', 'enqueue_scripts'));
		add_action('wp_ajax_secnin_get_visitors', array(__NAMESPACE__ . '\\Wf_Sn_Visitor_log', 'do_ajax_return_latest_visitors'));
		add_action('wp_ajax_secnin_vl_banip', array(__NAMESPACE__ . '\\Wf_Sn_Visitor_log', 'do_ajax_banip'));
	}


	/**
	 * URLs to be ignored when viewing the visitor log
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Wednesday, November 23rd, 2022.
	 * @access  public static
	 * @return  mixed
	 */
	public static function return_filterable_urls()
	{
		global $wpdb;
		$default_ignore   = array();
		$default_ignore[] = '/favicon.ico';
		$default_ignore[] = '/robots.txt';
		$default_ignore[] = '/?doing_wp_cron';
		$default_ignore[] = '/?kinsta-monitor'; // Filter out requests from Kinsta
		$default_ignore[] = '&mainwpsignature='; // Filter out requests from MainWP Dashboard
		$default_ignore[] = '/wordfence_lh='; // Filter out Wordfence requests
		$default_ignore[] = '/wp-json/'; // Filter out WordPress REST API requests
		$ignore_list      = apply_filters('securityninja_visitorlog_filter_url', $default_ignore);

		$clean_list = array();
		foreach ($ignore_list as $il) {
			$clean_list[] = $wpdb->remove_placeholder_escape(esc_sql($il));
		}
		return $clean_list;
	}


	/**
	 * Returns a row in HTML format ready to be displayed.
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Thursday, May 20th, 2021.
	 * @access  public static
	 * @param   mixed $visitor
	 * @param   mixed $fmtargs Default: array()
	 * @return  boolean
	 */
	public static function format_line($visitor, $fmtargs = array())
	{
		$defaults = array(
			'marknew' => true,
			'current_user_ip' => false,
		);
		$secnin_visitor_log_nonce = wp_create_nonce('secnin_visitor_log');
		$args = wp_parse_args($fmtargs, $defaults);

		$current_user_country = '';
		$ipcol_output = $visitor->ip;
		$geolocate_ip = \WPSecurityNinja\Plugin\SN_Geolocation::geolocate_ip($visitor->ip, true);
		if ($geolocate_ip) {
			$current_user_country = $geolocate_ip['country'];
			if ($current_user_country && '-' !== $current_user_country) {
				$country_img_url = wf_sn::get_country_img__premium_only($current_user_country);

				if (!isset($geoip_countrylist)) {
					include WF_SN_PLUGIN_DIR . 'modules/cloud-firewall/class-sn-geoip-countrylist.php';
				}
				$country_name = '';
				if (isset($geoip_countrylist[$current_user_country])) {
					$country_name = $geoip_countrylist[$current_user_country];
				}
				if ($country_img_url) {
					$ipcol_output .= '<img src="' . esc_url($country_img_url) . '" width="20" height="20" class="countryimg" title="' . esc_html($country_name) . '">';
				}
			}
		}




		if (!$visitor) {
			return false;
		}
		$rowatts = '';

		$rowclasses = '';

		if ($args['marknew']) {
			$rowclasses .= ' newrow';
		}
		$rowclasses .= ' visit-' . intval($visitor->id);

		$notes = '';
		if ($visitor->banned) {
			$rowclasses .= ' blocked';
			$notes      .= __('Blocked', 'security-ninja');
			if ($visitor->ban_reason) {
				$notes .= ': ' . esc_html($visitor->ban_reason);
			}
		}

		if (Wf_sn_cf::is_banned_ip($visitor->ip)) {
			$rowclasses .= ' blocked';
			$notes      .= __('Blacklisted by IP', 'security-ninja');
			if ($visitor->ban_reason) {
				$notes .= ': ' . esc_html($visitor->ban_reason);
			}
		}

		$rowatts = ' class="' . $rowclasses . '"';

		$details  = '<details><summary>' . __('Details', 'security-ninja') . '</summary><dl>';
		$details .= '<dt>' . __('User Agent', 'security-ninja') . '</dt><dd>' . esc_html($visitor->user_agent) . '</dd>';

		if ($visitor->action) {
			$details .= '<dt>' . __('Action', 'security-ninja') . '</dt><dd>' . esc_html($visitor->action) . '</dd>';
		}

		if (maybe_unserialize($visitor->description)) {
			$details .= '<dt>' . __('Description', 'security-ninja') . '</dt><dd>' . esc_html(print_r($visitor->description, true)) . '</dd>';
		}

		if (!empty($visitor->raw_data)) {
			$parsed_raw_data = json_decode($visitor->raw_data);
		} else {
			$parsed_raw_data = false;
		}

		if ((is_array($parsed_raw_data) || (is_object($parsed_raw_data))) && !empty($parsed_raw_data)) {
			$details .= '<dt>' . __('Details', 'security-ninja') . '</dt><dd>';
			$details .= '<dl class="inner">';
			foreach ($parsed_raw_data as $key => $rd) {
				$details .= '<dt>' . esc_html($key) . '</dt><dd>' . esc_html($rd) . '</dd>';
			}
			$details .= '</dl>';
			$details .= '</dd>';
		} elseif (is_string($parsed_raw_data)) {
			$details .= '<dt>' . __('Details', 'security-ninja') . '</dt><dd>' . esc_html($parsed_raw_data) . '</dd>';
		}

		$details .= '</dl></details>';

		$output       = '<tr ' . $rowatts . ' >';
		$output      .= '<td>' . esc_html($visitor->timestamp) . '</td>';
		$output      .= '<td>' . wp_kses_post($ipcol_output) . '</td>';
		$output      .= '<td>' . esc_url($visitor->URL) . '</td>';
		$allowed_html = wp_kses_allowed_html('post');
		$output      .= '<td class="secnin-details"><dd>' . wp_kses($details, $allowed_html) . '</dd></td>';

		if ($visitor->banned) {
			$output .= '<td></td>';
		} elseif ($args['current_user_ip'] === $visitor->ip) {
			$output .= '<td class="isme">' . __('You', 'security-ninja') . '</td>';
		} else {
			$output .= '<td class="secnin-visit-actions">';
			$output .= '<a href="#" data-banip="' . esc_attr($visitor->ip) . '" data-nonce="' . esc_attr($secnin_visitor_log_nonce) . '" class="button button-small button-secondary secnin-banip">' . __('Ban IP', 'security-ninja') . '</a>';

			$output .= '</td>';
		}

		$output .= '</tr>';

		return $output;
	}


	/**
	 * Enqueue CSS and JS scripts on plugin's admin page
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Tuesday, May 18th, 2021.
	 * @access  public static
	 * @global
	 * @return  void
	 */
	public static function enqueue_scripts()
	{
		$current_screen = get_current_screen();
		if (!$current_screen) {
			return false;
		}

		if (in_array(
			$current_screen->id,
			array(
				'security-ninja-1_page_wf-sn-visitor-log',
				'security-ninja-2_page_wf-sn-visitor-log',
				'security-ninja_page_wf-sn-visitor-log',
			),
			true
		)) {

			$js_vars = array(
				'vl_nonce' => wp_create_nonce('secnin_visitor_log'),
				'text'     => array(
					'notactive'         => __('This tab is not active, visitor data not updated', 'security-ninja'),
					'areyousureblockip' => __('Are you sure you want to block this IP?', 'security-ninja'),
				),
			);
			wp_enqueue_script('secnin-vl', WF_SN_PLUGIN_URL . 'modules/visitor-log/js/secnin-visitor-log-min.js', array('jquery'), Wf_Sn::$version);

			wp_localize_script('secnin-vl', 'secnin_vl', $js_vars);
		}
	}




	/**
	 * return_log_lines.
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Wednesday, November 23rd, 2022.
	 * @access  public static
	 * @param   mixed $ltestid
	 * @return  mixed
	 */
	public static function return_log_lines($latestid = false)
	{
		global $wpdb;

		$internal_ignore_list = self::return_filterable_urls();

		$ignore_list_string  = '';
		$ignore_list_string  = implode("%','", $internal_ignore_list);
		$ignore_list_string .= "', '";
		$ignore_list_string .= implode("','", $internal_ignore_list);

		$wherestring = '';
		$wherestring = "WHERE URL NOT IN ('%" . $ignore_list_string . "%')";

		if ($latestid) {
			$wherestring .= ' AND id > ' . $latestid;
		}

		$log_table    = $wpdb->prefix . WF_SN_CF_LOG_TABLE;
		$query_string = "SELECT * FROM {$log_table} {$wherestring} ORDER by id DESC LIMIT 100;";
		$results      = $wpdb->get_results($query_string, OBJECT);

		return $results;
	}



	/**
	 * return_latest_log_id.
	 *
	 * @author  Unknown
	 * @since   v0.0.1
	 * @version v1.0.0    Saturday, November 26th, 2022.
	 * @access  public static
	 * @global
	 * @param   integer $offset Default: 0
	 * @return  mixed
	 */
	public static function return_latest_log_id($offset = 0)
	{

		$internal_ignore_list = self::return_filterable_urls();

		$ignore_list_string  = '';
		$ignore_list_string  = implode("%','", $internal_ignore_list);
		$ignore_list_string .= "', '";
		$ignore_list_string .= implode("','", $internal_ignore_list);

		$wherestring = '';
		$wherestring = "WHERE URL NOT IN ('%" . $ignore_list_string . "%')";

		global $wpdb;
		$log_table = $wpdb->prefix . WF_SN_CF_LOG_TABLE;

		$offsetstring = '';
		if ($offset > 0) {
			$offsetstring = ' OFFSET ' . intval($offset);
		}

		$query_string = "SELECT id FROM {$log_table} {$wherestring} ORDER by id DESC LIMIT 1{$offsetstring};";
		$highestid    = $wpdb->get_var($query_string);

		if (!$highestid) {
			$highestid = 0;
		}
		return $highestid;
	}



	/**
	 * do_ajax_return_latest_visitors.
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Tuesday, May 18th, 2021.
	 * @version v1.0.1    Wednesday, November 23rd, 2022.
	 * @access  public static
	 * @return  void
	 */
	public static function do_ajax_return_latest_visitors()
	{

		if (!check_ajax_referer('secnin_visitor_log', false, false)) {
			wp_send_json_error(__('Invalid nonce', 'security-ninja'));
		}

		if (!current_user_can('manage_options')) {
			wp_send_json_error(
				array(
					'message' => __('Failed.', 'security-ninja'),
				)
			);
		}

		$latestid = isset($_GET['current']) ? sanitize_text_field($_GET['current']) : '';

		$results = self::return_log_lines($latestid);

		if ($results) {
			$highestid       = false;
			$latestvisits    = array();
			$current_user_ip = \WPSecurityNinja\Plugin\Wf_sn_cf::get_user_ip();
			$fmtlineargs     = array(
				'current_user_ip' => $current_user_ip,
			);
			foreach ($results as $re) {
				$highestid      = $re->id;
				$latestvisits[] = self::format_line($re, $fmtlineargs);
			}
			$latestvisits = array_reverse($latestvisits);
		}

		if ($results) {
			$output = array(
				'visits'  => $latestvisits,
				'current' => $highestid,
			);
			wp_send_json_success($output);
		} else {
			wp_send_json_error('No data');
		}
	}





	/**
	 * Ban IP via AJAX
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Thursday, November 11th, 2021.
	 * @access  public static
	 * @return  void
	 */
	public static function do_ajax_banip()
	{

		if (!check_ajax_referer('secnin_visitor_log', false, false)) {
			wp_send_json_error('Invalid nonce');
		}

		if (!current_user_can('manage_options')) {
			wp_send_json_error(
				array(
					'message' => __('Failed.', 'security-ninja'),
				)
			);
		}

		$banip = isset($_POST['banip']) ? sanitize_text_field($_POST['banip']) : false;

		if (!$banip) {
			wp_send_json_error(__('Forgot to say which IP?', 'security-ninja'));
		}

		if (!filter_var($banip, FILTER_VALIDATE_IP)) {
			wp_send_json_error(__('Please enter a valid IP.', 'security-ninja'));
		}

		if (!defined('WF_SN_CF_OPTIONS_KEY')) {
			return false;
		}

		$cf_options              = get_option(WF_SN_CF_OPTIONS_KEY);
		$blacklist               = $cf_options['blacklist'];
		$blacklist[]             = $banip;
		$blacklist               = array_unique($blacklist);
		$cf_options['blacklist'] = $blacklist;
		update_option(WF_SN_CF_OPTIONS_KEY, $cf_options, false);
		wp_send_json_success(__('Added IP to blacklist', 'security-ninja'));
		wf_sn_el_modules::log_event('security_ninja', 'blacklisted_ip', __('New IP added to the blacklist manually from the visitor log.', 'security-ninja'), '');
		return false;
	}



	/**
	 * Display the visitor log page
	 *
	 * @author  Lars Koudal
	 * @since   v0.0.1
	 * @version v1.0.0    Wednesday, May 19th, 2021.
	 * @version v1.0.1    Saturday, November 19th, 2022.
	 * @version v1.0.2    Wednesday, January 11th, 2023.
	 * @access  public static
	 * @return  void
	 */
	public static function live_log_page()
	{

		$options     = Wf_sn_cf::get_options();
		$trackvisits = intval($options['trackvisits']);

		if (!$trackvisits) {
?>
			<div class="secnin-live-vis">
				<p>Visitor log is not enabled. Go to the Firewall tab and turn on "Visitor Logging".</p>
			</div>
		<?php
		}

		?>
		<div class="wrap">
			<?php
			wf_sn::show_topbar();
			?>
			<div id="secnin-live-visitor-log-notice" class="">
				<h3><?php esc_html_e('Live Visitor Log', 'security-ninja'); ?></h3>
				<p><?php esc_html_e('Updates every 10 seconds when enabled.', 'security-ninja'); ?></p>
				<p><label for="secnin_enable_live">
						<input name="secnin_enable_live" type="checkbox" id="secnin_enable_live" value="1" checked><?php esc_html_e('Enable live updates', 'security-ninja'); ?></label></p>
			</div>
			<div id="enablecontainer">

				<p id="loadingspinner" class="spinner is-active"><span><?php esc_html_e('Loading', 'security-ninja'); ?></span></p>
				<div id="secnin-visitor-log-container">

					<div id="secnin-visitor-log-unfocused" style="display:none;">
						<h3><?php esc_html_e('Paused', 'security-ninja'); ?></h3>
						<p><?php esc_html_e('Switch back to this window to resume live updates.', 'security-ninja'); ?></p>
					</div><!-- #secnin-visitor-log-unfocused -->
					<table class="wp-list-table widefat flexible striped table-view-list" id="secnin-visitor-log">
						<thead>
							<tr>
								<th scope="col"><?php esc_html_e('Time', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('IP', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('URL', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('Details', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('Action', 'security-ninja'); ?></th>

							</tr>
						</thead>
						<tbody>
							<?php
							$results = self::return_log_lines(0);

							if ($results) {
								$current_user_ip = \WPSecurityNinja\Plugin\Wf_sn_cf::get_user_ip();
								$fmtlineargs     = array(
									'current_user_ip' => $current_user_ip,
								);
								foreach ($results as $re) {
									echo self::format_line($re, $fmtlineargs);
								}
							}

							$firstbatch = self::return_latest_log_id();

							?>
						</tbody>
						<tfoot>
							<tr>
								<th scope="col"><?php esc_html_e('Time', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('IP', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('URL', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('Details', 'security-ninja'); ?></th>
								<th scope="col"><?php esc_html_e('Action', 'security-ninja'); ?></th>
							</tr>
						</tfoot>
					</table>
				</div>
				<div id="secnin_vl_latestid"><?php echo esc_attr($firstbatch); ?></div>
			</div>
	<?php
	}
}

add_action('plugins_loaded', array(__NAMESPACE__ . '\Wf_Sn_Visitor_log', 'init'));
